import { MongoClient } from 'mongodb';
const fs = require('fs');

module.exports = (on: any, config: any) => {
  on('after:spec', (spec: Cypress.Spec, results: CypressCommandLine.RunResult) => {
    if (results && results.video && results.stats.failures === 0) {
      console.log(`Delete recorded video because spec passed: ${results.video}`);
      fs.unlinkSync(results.video);
    }
  });

  // Workaround für Angular 13 und Cypress mit Webpack 4,
  // Siehe https://github.com/cypress-io/cypress/issues/19066#issuecomment-1012055705
  // Entfernen, sobald Cypress Webpack 5 nutzt - https://github.com/cypress-io/cypress/issues/19555
  // Ursache: Angular linker needed to link partial-ivy code,
  //   see https://angular.io/guide/creating-libraries#consuming-partial-ivy-code-outside-the-angular-cli
  // Fehlerbild:
  //   - Anwendung läuft im Browser, aber nicht in Cypress.
  //   - Fehlermeldung in Cypress: The injectable 'SystemDateTimeProvider' needs to be compiled using the JIT compiler, but '@angular/compiler' is not available.
  // Lösung:
  //   - NPM-Paket identifizieren, dass "SystemDateTimeProvider" enthält.
  //   - NPM-Paket im "test" Attribut unten hinzufügen.
  const webpackPreprocessor = require('@cypress/webpack-batteries-included-preprocessor');
  const webpackOptions = webpackPreprocessor.defaultOptions.webpackOptions;

  webpackOptions.module.rules.unshift({
    test: /[/\\](@angular|@ngxp|angular-oauth2-oidc)[/\\].+\.m?js$/,
    resolve: {
      fullySpecified: false,
    },
    use: {
      loader: 'babel-loader',
      options: {
        plugins: ['@angular/compiler-cli/linker/babel'],
        compact: false,
        cacheDirectory: true,
      },
    },
  });

  on(
    'file:preprocessor',
    webpackPreprocessor({
      webpackOptions: webpackOptions,
      typescript: require.resolve('typescript'),
    }),
  );

  on('task', {
    dropCollections(collections) {
      console.log('dropCollections: ', collections);
      dropCollectionsFromDatabase(config, collections);
      return 0;
    },
  });

  return config;
  // Ende - Workaround für Angular 13 und Cypress mit Webpack 4
};

function dropCollectionsFromDatabase(config, collections) {
  dropCollections(getDatabaseUrl(config), getDatabase(config), collections);
}

function getDatabaseUrl(config) {
  return config.env.dbUrl;
}

function getDatabase(config) {
  return config.env.database;
}

function dropCollections(databaseUrl, databaseName, collections) {
  MongoClient.connect(databaseUrl, (error, connection) => {
    if (!error) {
      const db = connection.db(databaseName);
      collections.forEach((oneCollection, index) => {
        console.log('drop collection', oneCollection);
        db.collection(oneCollection).drop(() => {
          //CHECKME Ist die Abfrage notwendig?
          if (index == collections.length) {
            console.log('close connection');
            connection.close();
          }
        });
      });
    } else {
      console.error('Error: ', error);
    }
  });
}
