import { Injectable } from '@angular/core';
import KcAdminClient from '@keycloak/keycloak-admin-client';
import { TokenProvider } from '@keycloak/keycloak-admin-client/lib/client';
import GroupRepresentation from '@keycloak/keycloak-admin-client/lib/defs/groupRepresentation';
import MappingsRepresentation from '@keycloak/keycloak-admin-client/lib/defs/mappingsRepresentation';
import RoleRepresentation from '@keycloak/keycloak-admin-client/lib/defs/roleRepresentation';
import UserRepresentation from '@keycloak/keycloak-admin-client/lib/defs/userRepresentation';
import { OAuthService } from 'angular-oauth2-oidc';
import { isNil } from 'lodash-es';
import { Observable, forkJoin, from, map, mergeMap } from 'rxjs';
import { User } from './user.model';

@Injectable({
  providedIn: 'root',
})
export class UserRepository {
  constructor(
    private oAuthService: OAuthService,
    private kcAdminClient: KcAdminClient,
  ) {
    this.registerAccessTokenProvider();
  }

  private registerAccessTokenProvider(): void {
    this.kcAdminClient.registerTokenProvider(this.getTokenProvider());
  }

  private getTokenProvider(): TokenProvider {
    return {
      getAccessToken: () => Promise.resolve(this.oAuthService.getAccessToken()),
    };
  }

  public getUsers(): Observable<User[]> {
    return from(this.kcAdminClient.users.find()).pipe(
      map((userReps: UserRepresentation[]) => userReps.map((userReps) => this.mapToUser(userReps))),
      mergeMap((users) => forkJoin(users.map((users) => this.addInformationToUser(users)))),
    );
  }

  mapToUser(userRepresentation: UserRepresentation): User {
    return {
      id: userRepresentation.id,
      email: userRepresentation.email,
      username: userRepresentation.username,
      firstName: userRepresentation.firstName,
      lastName: userRepresentation.lastName,
      groups: null,
      roles: null,
    };
  }

  private addInformationToUser(user: User): Observable<User> {
    return forkJoin([this.getUserGroups(user), this.getAlfaClientRoles(user)]).pipe(
      map(([groups, roles]) => ({
        ...user,
        groups,
        roles,
      })),
    );
  }

  private getUserGroups(user: User): Observable<string[]> {
    return from(this.kcAdminClient.users.listGroups({ id: user.id })).pipe(
      map((groups: GroupRepresentation[]) => groups.map((group) => group.name)),
    );
  }

  private getAlfaClientRoles(user: User): Observable<string[]> {
    return from(this.kcAdminClient.users.listRoleMappings({ id: user.id })).pipe(
      map((clientMappings: MappingsRepresentation) => this.mapToAlfaClientRoleNames(clientMappings)),
    );
  }

  private mapToAlfaClientRoleNames(roleMappings: MappingsRepresentation): string[] {
    if (isNil(roleMappings?.clientMappings?.['alfa'])) {
      return [];
    }

    return roleMappings.clientMappings['alfa'].mappings.map((role: RoleRepresentation) => role.name);
  }
}
