import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { UserProfileResource } from '@alfa-client/user-profile-shared';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { AuthConfig, OAuthService } from 'angular-oauth2-oidc';
import { AuthenticationService } from './authentication.service';
import { createAuthConfig } from '../../test/authentication';
import { createEnvironment } from '../../../environment-shared/test/environment';
import { Environment } from 'libs/environment-shared/src/lib/environment.model';

describe('AuthenticationService', () => {
  let service: AuthenticationService;
  let oAuthService: Mock<OAuthService>;
  let environmentConfig: Environment;

  beforeEach(() => {
    oAuthService = <any>{
      ...mock(OAuthService),
      loadDiscoveryDocumentAndLogin: jest.fn().mockResolvedValue(() => Promise.resolve()),
    };
    environmentConfig = createEnvironment();
    service = new AuthenticationService(useFromMock(oAuthService), environmentConfig);
  });

  describe('login', () => {
    beforeEach(() => {
      service.setCurrentUser = jest.fn();
    });

    it('should configure service with authConfig', async () => {
      const authConfig: AuthConfig = createAuthConfig();
      service.buildConfiguration = jest.fn().mockReturnValue(authConfig);

      await service.login();

      expect(oAuthService.configure).toHaveBeenCalledWith(authConfig);
    });

    it('should setup automatic silent refresh', async () => {
      await service.login();

      expect(oAuthService.setupAutomaticSilentRefresh).toHaveBeenCalled();
    });

    it('should set tokenValidator', async () => {
      oAuthService.tokenValidationHandler = null;

      await service.login();

      expect(oAuthService.tokenValidationHandler).not.toBeNull();
    });

    it('should load discovery document and login', () => {
      service.login();

      expect(oAuthService.loadDiscoveryDocumentAndLogin).toHaveBeenCalled();
    });

    it('should set current user', async () => {
      await service.login();

      expect(service.setCurrentUser).toHaveBeenCalled();
    });
  });

  describe('set current user', () => {
    const identityClaims: Record<string, any> = {
      ['given_name']: 'Marco',
      ['family_name']: 'Polo',
    };

    beforeEach(() => {
      oAuthService.getIdentityClaims.mockReturnValue(identityClaims);
    });
    it('should call oAuthservice to get claims', () => {
      service.setCurrentUser();

      expect(oAuthService.getIdentityClaims).toHaveBeenCalled();
    });

    it('should update currentUser', () => {
      service.setCurrentUser();

      expect(service.currentUserResource.firstName).toEqual('Marco');
      expect(service.currentUserResource.lastName).toEqual('Polo');
    });
  });

  describe('getCurrentUserInitials', () => {
    it('should return currentUserResource', () => {
      const userProfile: UserProfileResource = {
        ...createUserProfileResource(),
        firstName: 'Marco',
        lastName: 'Polo',
      };
      service.currentUserResource = userProfile;

      const currentUserInitials: string = service.getCurrentUserInitials();

      expect(currentUserInitials).toEqual('MP');
    });
  });

  describe('logout', () => {
    it('should call oAuthService revokeTokenAndLogout', () => {
      service.logout();

      expect(oAuthService.revokeTokenAndLogout).toHaveBeenCalled();
    });
  });
});
