import { CommandOrder, CommandResource, CommandService } from '@alfa-client/command-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { createCollaboration, createCollaborationListResource } from '../../../collaboration-shared/test/collaboration';
import { CollaborationListResourceService } from './collaboration-list-resource.service';
import { CollaborationListLinkRel } from './collaboration.linkrel';
import { Collaboration, CollaborationListResource } from './collaboration.model';
import { CollaborationService } from './collaboration.service';

jest.mock('./collaboration-list-resource.service');

describe('CollaborationService', () => {
  let service: CollaborationService;

  let listService: Mock<CollaborationListResourceService>;
  let commandService: Mock<CommandService>;

  const collaborationListResource: CollaborationListResource = createCollaborationListResource();
  const collaborationStateListResource: StateResource<CollaborationListResource> = createStateResource(collaborationListResource);

  beforeEach(() => {
    listService = mock(CollaborationListResourceService);
    commandService = mock(CommandService);

    service = new CollaborationService(useFromMock(listService), useFromMock(commandService));
  });

  describe('get list', () => {
    beforeEach(() => {
      listService.getList.mockReturnValue(of(collaborationStateListResource));
    });

    it('should call service', () => {
      service.getList();

      expect(listService.getList).toHaveBeenCalled();
    });

    it('should return value', () => {
      const list$: Observable<StateResource<CollaborationListResource>> = service.getList();

      expect(list$).toBeObservable(singleColdCompleted(collaborationStateListResource));
    });
  });

  describe('organisationsEinheit', () => {
    describe('is form visible', () => {
      it('should return value', (done) => {
        service.showOrganisationsEinheitForm$.next(true);

        service.isOrganisationsEinheitFormVisible().subscribe((isVisible: boolean) => {
          expect(isVisible).toBeTruthy();
          done();
        });

        service.isOrganisationsEinheitFormVisible();
      });
    });

    describe('show formular', () => {
      it('should set form visibility to true', () => {
        service.showOrganisationsEinheitForm$.next(false);

        service.showOrganisationsEinheitForm();

        expect(service.showOrganisationsEinheitForm$.value).toBeTruthy();
      });

      it('should disable externeFachstelle button visibility', () => {
        service.showExterneFachstelleAnfrageButton$.next(true);

        service.showOrganisationsEinheitForm();

        expect(service.showExterneFachstelleAnfrageButton$.value).toBeFalsy();
      });

      it('should set collaborationLevel to 1', () => {
        service.showOrganisationsEinheitForm();

        expect(service.collaborationLevel$.value).toEqual(1);
      });
    });

    describe('hide formular', () => {
      it('should set "showRequestForm" to false', () => {
        service.showOrganisationsEinheitForm$.next(true);

        service.hideOrganisationseinheitForm();

        expect(service.showOrganisationsEinheitForm$.value).toBeFalsy();
      });

      it('should enable externe fachstelle button visibility', () => {
        service.showExterneFachstelleAnfrageButton$.next(false);

        service.hideOrganisationseinheitForm();

        expect(service.showExterneFachstelleAnfrageButton$.value).toBeTruthy();
      });

      it('should reset collaborationLevel to null', () => {
        service.collaborationLevel$.next(1);

        service.hideOrganisationseinheitForm();

        expect(service.collaborationLevel$.value).toBeNull();
      });
    });

    describe('show anfrage button', () => {
      it('should return value', (done) => {
        service.showOrganisationsEinheitAnfrageButton$.next(true);

        service.showOrganisationsEinheitAnfrageButton().subscribe((isVisible: boolean) => {
          expect(isVisible).toBeTruthy();
          done();
        });

        service.showOrganisationsEinheitAnfrageButton();
      });
    });
  });

  describe('externe fachstelle', () => {
    describe('is form visible', () => {
      it('should return value', (done) => {
        service.showExterneFachstelleForm$.next(true);

        service.isExterneFachstelleFormVisible().subscribe((isVisible: boolean) => {
          expect(isVisible).toBeTruthy();
          done();
        });

        service.isExterneFachstelleFormVisible();
      });
    });

    describe('show formular', () => {
      it('should set form visbility to true', () => {
        service.showExterneFachstelleForm$.next(false);

        service.showExterneFachstelleForm();

        expect(service.showExterneFachstelleForm$.value).toBeTruthy();
      });

      it('should disable organisationsEinheit button visibility', () => {
        service.showOrganisationsEinheitAnfrageButton$.next(true);

        service.showExterneFachstelleForm();

        expect(service.showOrganisationsEinheitAnfrageButton$.value).toBeFalsy();
      });

      it('should set collaborationLevel to 4', () => {
        service.showExterneFachstelleForm();

        expect(service.collaborationLevel$.value).toEqual(4);
      });
    });

    describe('hide formular', () => {
      it('should set "showRequestForm" to false', () => {
        service.showExterneFachstelleForm$.next(true);

        service.hideExterneFachstelleForm();

        expect(service.showExterneFachstelleForm$.value).toBeFalsy();
      });

      it('should enable organisationsEinheit button visibility', () => {
        service.showOrganisationsEinheitAnfrageButton$.next(false);

        service.hideExterneFachstelleForm();

        expect(service.showOrganisationsEinheitAnfrageButton$.value).toBeTruthy();
      });

      it('should reset collaborationLevel to null', () => {
        service.collaborationLevel$.next(4);

        service.hideExterneFachstelleForm();

        expect(service.collaborationLevel$.value).toBeNull();
      });
    });

    describe('show anfrage button', () => {
      it('should return value', (done) => {
        service.showExterneFachstelleAnfrageButton$.next(true);

        service.showExterneFachstelleAnfrageButton().subscribe((isVisible: boolean) => {
          expect(isVisible).toBeTruthy();
          done();
        });

        service.showExterneFachstelleAnfrageButton();
      });
    });
  });

  describe('create', () => {
    const collaborationListResource: CollaborationListResource = createCollaborationListResource();
    const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());
    const collaboration: Collaboration = createCollaboration();

    beforeEach(() => {
      commandService.createCommandByProps.mockReturnValue(of(commandStateResource));
      service.collaborationLevel$.next(1);
    });

    it('should call command service', () => {
      service.create(collaborationListResource, collaboration);

      expect(commandService.createCommandByProps).toHaveBeenCalledWith({
        linkRel: CollaborationListLinkRel.CREATE_COLLABORATION_REQUEST,
        resource: collaborationListResource,
        command: {
          order: CommandOrder.CREATE_COLLABORATION_REQUEST,
          body: { ...collaboration, collaborationLevel: 1 },
        },
        snackBarMessage: 'Die Zuarbeit wurde angefragt.',
      });
    });

    it('should return value', () => {
      const created$: Observable<StateResource<CommandResource>> = service.create(collaborationListResource, collaboration);

      expect(created$).toBeObservable(singleColdCompleted(commandStateResource));
    });
  });
});
