/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { faker } from '@faker-js/faker';
import { mock, useFromMock } from '@alfa-client/test-utils';
import { Resource, ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import {
  createCommand,
  createCommandListResource,
  createCommandResource,
} from 'libs/command-shared/test/command';
import { toResource } from 'libs/tech-shared/test/resource';
import { CommandLinkRel } from './command.linkrel';
import { Command, CommandListResource, CommandResource, CommandStatus } from './command.model';
import { CommandRepository } from './command.repository';

describe('CommandRepository', () => {
  let repository: CommandRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { post: jest.fn(), get: jest.fn(), patch: jest.fn() };

  const command: Command = createCommand();
  const commandResource: CommandResource = createCommandResource();
  const commandListResource: CommandListResource = createCommandListResource();
  const commandResourceWithPendingCommand: CommandResource = createCommandResource([
    'pendingCommands',
  ]);

  beforeEach(() => {
    repository = new CommandRepository(useFromMock(resourceFactory));
    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('create command', () => {
    const linkRel: string = faker.word.sample();
    const resource: Resource = toResource({}, [linkRel]);

    beforeEach(() => {
      resourceWrapper.post.mockReturnValue(hot('a', { a: commandResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.createCommand(resource, linkRel, command);

      expect(resourceFactory.from).toHaveBeenCalledWith(resource);
    });

    it('should call resourceWrapper with linkel and command', () => {
      repository.createCommand(resource, linkRel, command);

      expect(resourceWrapper.post).toHaveBeenCalledWith(linkRel, command);
    });

    it('should return value', () => {
      const expected: CommandResource = commandResource;

      const result = repository.createCommand(resource, linkRel, command);

      expect(result).toBeObservable(cold('a', { a: expected }));
    });
  });

  describe('get command', () => {
    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: commandResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.getCommand(commandResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(commandResource);
    });

    it('should call resourceWrapper with selfLink', () => {
      repository.getCommand(commandResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith(CommandLinkRel.SELF);
    });

    it('should return value', () => {
      const result = repository.getCommand(commandResource);

      expect(result).toBeObservable(cold('a', { a: commandResource }));
    });
  });

  describe('revoke command', () => {
    beforeEach(() => {
      resourceWrapper.patch.mockReturnValue(hot('a', { a: commandResource }));
    });

    it('should call resourceFactory with Resource', () => {
      repository.revokeCommand(commandResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(commandResource);
    });

    it('should call resourceWrapper with revokeLinkRel and command order', () => {
      repository.revokeCommand(commandResource);

      expect(resourceWrapper.patch).toHaveBeenCalledWith(CommandLinkRel.REVOKE, {
        status: CommandStatus.REVOKED,
      });
    });

    it('should return value', () => {
      const result = repository.revokeCommand(commandResource);

      expect(result).toBeObservable(cold('a', { a: commandResource }));
    });
  });

  describe('get pending commands', () => {
    const linkRel = 'pendingCommands';

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: commandListResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.getPendingCommands(commandResourceWithPendingCommand, linkRel);

      expect(resourceFactory.from).toHaveBeenCalledWith(commandResourceWithPendingCommand);
    });

    it('should call resourceWrapper with linkRel', () => {
      repository.getPendingCommands(commandResourceWithPendingCommand, linkRel);

      expect(resourceWrapper.get).toHaveBeenCalledWith(linkRel);
    });

    it('should return value', () => {
      const result = repository.getPendingCommands(commandResourceWithPendingCommand, linkRel);

      expect(result).toBeObservable(cold('a', { a: commandListResource }));
    });
  });

  describe('get effected resource', () => {
    const command: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);
    const listResource = {};

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: listResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.getEffectedResource(command);

      expect(resourceFactory.from).toHaveBeenCalledWith(command);
    });

    it('should call resourceWrapper with linkRel', () => {
      repository.getEffectedResource(command);

      expect(resourceWrapper.get).toHaveBeenCalledWith(CommandLinkRel.EFFECTED_RESOURCE);
    });

    it('should return value', () => {
      const result = repository.getEffectedResource(command);

      expect(result).toBeObservable(cold('a', { a: listResource }));
    });
  });
});
