/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Injectable, OnDestroy } from '@angular/core';
import { Params } from '@angular/router';
import {
  CommandResource,
  CommandService,
  CreateCommand,
  isDone,
  isPending,
} from '@alfa-client/command-shared';
import { NavigationService } from '@alfa-client/navigation-shared';
import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
  hasStateResourceError,
} from '@alfa-client/tech-shared';
import {
  ForwardRequest,
  VorgangResource,
  VorgangService,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
  createForwardCommand,
} from '@alfa-client/vorgang-shared';
import { isNil } from 'lodash-es';
import { BehaviorSubject, Observable, Subscription } from 'rxjs';
import { first, map, startWith, tap } from 'rxjs/operators';
import { ForwardingLinkRel } from './forwarding.linkrel';
import { ForwardingListResource, ForwardingResource } from './forwarding.model';
import { ForwardingRepository } from './forwarding.repository';
import {
  createForwardingMarkAsFailCommand,
  createForwardingMarkAsSuccessCommand,
} from './forwarding.util';

@Injectable({ providedIn: 'root' })
export class ForwardingService implements OnDestroy {
  private readonly forwardingList$: BehaviorSubject<StateResource<ForwardingListResource>> =
    new BehaviorSubject(createEmptyStateResource<ForwardingListResource>());

  private navigationSubscription: Subscription;
  private subscription: Subscription;

  constructor(
    private commandService: CommandService,
    private vorgangService: VorgangService,
    private navigationService: NavigationService,
    private forwardingRepository: ForwardingRepository,
  ) {}

  getForwardings(
    vorgang: VorgangWithEingangResource,
  ): Observable<StateResource<ForwardingListResource>> {
    this.forwardingList$.next(createEmptyStateResource(true));

    const subscription = this.forwardingRepository.getForwardings(vorgang).subscribe((list) => {
      this.forwardingList$.next(createStateResource(list));
      subscription.unsubscribe();
    });

    return this.forwardingList$.asObservable();
  }

  getPendingForwardCommand(): Observable<StateResource<CommandResource>> {
    this.listenToNavigation();

    return this.vorgangService
      .getPendingForwardCommand()
      .pipe(map((pendingCommand) => this.pollPendingForwardCommand(pendingCommand)));
  }

  listenToNavigation(): void {
    this.unsubscribe();
    this.navigationSubscription = this.navigationService
      .urlChanged()
      .subscribe((params) => this.onNavigation(params));
  }

  onNavigation(params: Params): void {
    if (NavigationService.isVorgangListPage(params)) {
      this.unsubscribe();
    }
  }

  pollPendingForwardCommand(
    command: StateResource<CommandResource>,
  ): StateResource<CommandResource> {
    if (this.shouldPoll(command)) {
      this.subscription = this.commandService
        .pollCommand(command.resource)
        .subscribe((updatedCommand) => {
          if (isDone(updatedCommand.resource)) {
            this.vorgangService.setPendingForwardSingleCommand(updatedCommand);
            this.reloadCurrentVorgang(updatedCommand);
            this.subscription.unsubscribe();
          }
        });
    }
    return command;
  }

  private shouldPoll(command: StateResource<CommandResource>): boolean {
    return command.loaded && isPending(command.resource);
  }

  public forward(
    vorgang: VorgangResource,
    request: ForwardRequest,
  ): Observable<StateResource<CommandResource>> {
    this.vorgangService.setPendingForwardSingleCommandLoading();
    const subscription: Subscription = this.commandService
      .createCommand(vorgang, VorgangWithEingangLinkRel.FORWARD, createForwardCommand(request))
      .subscribe((command) => {
        this.vorgangService.setPendingForwardSingleCommand(command);
        this.reloadCurrentVorgang(command);
        subscription.unsubscribe();
      });
    return this.vorgangService.getPendingForwardCommand();
  }

  markAsSuccess(forward: ForwardingResource): Observable<StateResource<CommandResource>> {
    return this.createMarkAsCommand(
      forward,
      ForwardingLinkRel.MARK_AS_SUCCESS,
      createForwardingMarkAsSuccessCommand(),
    ).pipe(tap((command) => this.updateForwardingList(command)));
  }

  markAsFail(forward: ForwardingResource): Observable<StateResource<CommandResource>> {
    return this.createMarkAsCommand(
      forward,
      ForwardingLinkRel.MARK_AS_FAIL,
      createForwardingMarkAsFailCommand(),
    ).pipe(
      tap((command) => {
        if (isDone(command.resource)) this.reloadCurrentVorgang(command);
      }),
    );
  }

  reloadCurrentVorgang(command: StateResource<CommandResource>): void {
    if (!hasStateResourceError(command)) {
      this.vorgangService.reloadCurrentVorgang();
    }
  }

  createMarkAsCommand(
    forward: ForwardingResource,
    linkRel: string,
    command: CreateCommand,
  ): Observable<StateResource<CommandResource>> {
    return this.commandService
      .createCommand(forward, linkRel, command)
      .pipe(startWith(createEmptyStateResource<CommandResource>(true)));
  }

  updateForwardingList(command: StateResource<CommandResource>): void {
    if (isDone(command.resource)) this.updateList(command.resource);
  }

  updateList(command: CommandResource): void {
    this.commandService
      .getEffectedResource<ForwardingListResource>(command)
      .pipe(first())
      .subscribe((list) => this.setList(list));
  }

  setList(list: ForwardingListResource): void {
    this.forwardingList$.next(createStateResource(list));
  }

  ngOnDestroy(): void {
    this.unsubscribe();
  }

  unsubscribe(): void {
    if (!isNil(this.navigationSubscription)) this.navigationSubscription.unsubscribe();
    if (!isNil(this.subscription)) this.subscription.unsubscribe();
  }
}
