import { EMPTY_ARRAY, StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { faker } from '@faker-js/faker';
import { InstantSearchResult } from '@ods/system';
import { createInstantSearchResult } from 'libs/design-system/src/test/search';
import { createEmptyListResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { singleColdCompleted } from '../../../../tech-shared/test/marbles';
import { createOrganisationsEinheitListResource, createOrganisationsEinheitResource } from '../../../test/organisations-einheit';
import { OrganisationsEinheitResourceSearchService } from './organisations-einheit-resource-search.service';
import { OrganisationsEinheitListResource, OrganisationsEinheitResource } from './organisations-einheit.model';
import { OrganisationsEinheitService } from './organisations-einheit.service';

jest.mock('./organisations-einheit-resource-search.service');

describe('OrganisationsEinheitService', () => {
  let service: OrganisationsEinheitService;

  let searchService: Mock<OrganisationsEinheitResourceSearchService>;

  const listResource: OrganisationsEinheitListResource = createOrganisationsEinheitListResource();
  const listStateResource: StateResource<OrganisationsEinheitListResource> = createStateResource(listResource);

  beforeEach(() => {
    searchService = mock(OrganisationsEinheitResourceSearchService);

    service = new OrganisationsEinheitService(useFromMock(searchService));
  });

  describe('get search result list', () => {
    const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> = createInstantSearchResult();

    beforeEach(() => {
      searchService.getResultList.mockReturnValue(of(listStateResource));
      service.mapListResource = jest.fn().mockReturnValue([instantSearchResult]);
    });

    it('should call search service', () => {
      service.getSearchResultList();

      expect(searchService.getResultList).toHaveBeenCalled();
    });

    it('should map list resource', () => {
      service.getSearchResultList().subscribe();

      expect(service.mapListResource).toHaveBeenCalledWith(listStateResource);
    });

    it('should return result', (done) => {
      searchService.getResultList.mockReturnValue(of(listStateResource));

      service.getSearchResultList().subscribe((result: InstantSearchResult<OrganisationsEinheitResource>[]) => {
        expect(result).toEqual([instantSearchResult]);
        done();
      });
    });
  });

  describe('map listResource', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();
    const singleEntrylistResource: OrganisationsEinheitListResource = createOrganisationsEinheitListResource([
      organisationsEinheitResource,
    ]);
    const singleEntryListStateResource: StateResource<OrganisationsEinheitListResource> =
      createStateResource(singleEntrylistResource);

    const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> = createInstantSearchResult();

    beforeEach(() => {
      service.mapToInstantSearchResult = jest.fn().mockReturnValue(instantSearchResult);
    });

    it('should map to instantSearchResult', () => {
      service.mapListResource(singleEntryListStateResource);

      expect(service.mapToInstantSearchResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });

    it('should return value', () => {
      const result: InstantSearchResult<OrganisationsEinheitResource>[] = service.mapListResource(singleEntryListStateResource);

      expect(result).toEqual([instantSearchResult]);
    });

    it('should return empty array on empty listResource', () => {
      const result: InstantSearchResult<OrganisationsEinheitResource>[] = service.mapListResource(
        createStateResource(createEmptyListResource<OrganisationsEinheitListResource>()),
      );

      expect(result).toEqual(EMPTY_ARRAY);
    });
  });

  describe('map to instant search result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

    it('should map titel', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        service.mapToInstantSearchResult(organisationsEinheitResource);

      expect(instantSearchResult.title).toBe(organisationsEinheitResource.name);
    });
    it('should map description', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        service.mapToInstantSearchResult(organisationsEinheitResource);

      const expectedDescription: string = `${organisationsEinheitResource.anschrift.strasse} ${organisationsEinheitResource.anschrift.hausnummer}, ${organisationsEinheitResource.anschrift.plz} ${organisationsEinheitResource.anschrift.ort}`;
      expect(instantSearchResult.description).toBe(expectedDescription);
    });

    it('should map data', () => {
      const instantSearchResult: InstantSearchResult<OrganisationsEinheitResource> =
        service.mapToInstantSearchResult(organisationsEinheitResource);

      expect(instantSearchResult.data).toBe(organisationsEinheitResource);
    });
  });

  describe('search', () => {
    const searchBy: string = faker.lorem.word();

    it('should call search service with search string', () => {
      service.search(searchBy);

      expect(searchService.search).toHaveBeenCalledWith(searchBy);
    });
  });

  describe('clear search result', () => {
    it('should call search service', () => {
      service.clearSearchResult();

      expect(searchService.clearResultList).toHaveBeenCalledWith();
    });
  });

  describe('get selected result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

    beforeEach(() => {
      searchService.getSelectedResult.mockReturnValue(of(organisationsEinheitResource));
    });

    it('should call service', () => {
      service.getSelectedResult();

      expect(searchService.getSelectedResult).toHaveBeenCalled();
    });

    it('should return result', () => {
      const selectedResult$: Observable<OrganisationsEinheitResource> = service.getSelectedResult();

      expect(selectedResult$).toBeObservable(singleColdCompleted(organisationsEinheitResource));
    });
  });

  describe('select search result', () => {
    const organisationsEinheitResource: OrganisationsEinheitResource = createOrganisationsEinheitResource();

    it('should call service', () => {
      service.selectSearchResult(organisationsEinheitResource);

      expect(searchService.selectResult).toHaveBeenCalledWith(organisationsEinheitResource);
    });
  });

  describe('clear selected result', () => {
    it('should call service', () => {
      service.clearSelectedResult();

      expect(searchService.clearSelectedResult).toHaveBeenCalled();
    });
  });
});
