package de.ozgcloud.alfa.loeschanforderung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.Test;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.common.command.CommandTestFactory;

class LoeschAnforderungCommandProcessorTest {

	@Spy
	@InjectMocks
	private LoeschAnforderungCommandProcessor processor;

	@Mock
	private LoeschAnforderungService service;

	@DisplayName("Prozess")
	@Nested
	class TestProcess {

		@BeforeEach
		void mock() {
			UserProfileUrlProviderTestFactory.initUserProfileUrlProvider(new UserProfileUrlProvider());
		}

		@DisplayName("on revokeable")
		@Nested
		class TestRevokeableCommand {

			@DisplayName("zum loeschen markieren command")
			@Nested
			class TestLoeschAnforderungCommand {

				private final Command finishedCommand = CommandTestFactory.createBuilder().status(CommandStatus.FINISHED).build();

				@BeforeEach
				void mock() {
					when(service.findLoeschAnforderung(any())).thenReturn(Optional.of(VorgangAttachedItemTestFactory.create()));
				}

				@Test
				void shouldAddLink() {
					var entityModel = EntityModel
							.of(finishedCommand.toBuilder().order(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN.name()).build());

					var prozessedCommand = processor.process(entityModel);

					assertThat(prozessedCommand.getLink(LoeschAnforderungCommandProcessor.REL_REVOKE)).isPresent().get().extracting(Link::getHref)
							.isEqualTo("/api/loeschanforderungs/" + VorgangAttachedItemTestFactory.ID + "/commands/" + CommandTestFactory.ID);
				}

				@Test
				void shouldCallService() {
					var entityModel = EntityModel
							.of(finishedCommand.toBuilder().order(CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN.name()).build());

					var prozessedCommand = processor.process(entityModel);

					assertThat(prozessedCommand.getLink(LoeschAnforderungCommandProcessor.REL_REVOKE)).isPresent().get().extracting(Link::getHref)
							.isEqualTo("/api/loeschanforderungs/" + VorgangAttachedItemTestFactory.ID + "/commands/" + CommandTestFactory.ID);
				}
			}

			@DisplayName("loeschAnforderung zuruecknehmen command")
			@Nested
			class TestLoeschAnforderungZuruecknehmenCommand {

				private final Command finishedCommand = CommandTestFactory.createBuilder().status(CommandStatus.FINISHED).build();

				@Test
				void shouldAddLink() {
					var entityModel = EntityModel
							.of(finishedCommand.toBuilder().order(CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN.name()).build());

					var prozessedCommand = processor.process(entityModel);

					assertThat(prozessedCommand.getLink(LoeschAnforderungCommandProcessor.REL_REVOKE)).isPresent().get().extracting(Link::getHref)
							.isEqualTo("/api/loeschanforderungs/" + CommandTestFactory.RELATION_ID + "/commands/" + CommandTestFactory.ID);
				}
			}

			@DisplayName("on other commands")
			@Nested
			class TestOnOtherCommand {

				private final Command finishedCommand = CommandTestFactory.createBuilder().status(CommandStatus.FINISHED).build();

				@DisplayName("except zum loeschen markieren")
				@Nested
				class TestsExceptLoeschAnforderungZuruecknehmen {

					@ParameterizedTest
					@EnumSource(mode = Mode.EXCLUDE, names = { "VORGANG_ZUM_LOESCHEN_MARKIEREN" })
					void shouldNotCallService(CommandOrder order) {
						var command = finishedCommand.toBuilder().order(order.name()).build();

						processor.process(EntityModel.of(command));

						verify(service, never()).findLoeschAnforderung(any());
					}
				}

				@ParameterizedTest
				@EnumSource(mode = Mode.EXCLUDE, names = { "VORGANG_ZUM_LOESCHEN_MARKIEREN", "LOESCH_ANFORDERUNG_ZURUECKNEHMEN" })
				void NotAddRevokeLink(CommandOrder order) {
					var command = finishedCommand.toBuilder().order(order.name()).build();

					var prozessedCommand = processor.process(EntityModel.of(command));

					assertThat(prozessedCommand.getLink(LoeschAnforderungCommandProcessor.REL_REVOKE)).isEmpty();
				}
			}
		}
	}
}