/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Optional;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class WiedervorlageServiceTest {

	@Spy
	@InjectMocks
	private WiedervorlageService service;
	@Mock
	private WiedervorlageRemoteService remoteService;
	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;
	@Mock
	private CommandService commandService;
	@Mock
	private CurrentUserService currentUserService;

	@DisplayName("Create wiedervorlage")
	@Nested
	class TestCreateWiedervorlage {

		@DisplayName("should")
		@Nested
		class TestCalls {

			@BeforeEach
			void mockServices() {
				doReturn(WiedervorlageTestFactory.create()).when(service).addCreated(any());
				when(vorgangAttachedItemService.createNewWiedervorlage(any(), any())).thenReturn(CommandTestFactory.create());
			}

			@DisplayName("add created")
			@Test
			void shouldAddCreated() {
				callCreateWiedervorlage();

				verify(service).addCreated(any(Wiedervorlage.class));
			}

			@DisplayName("call vorgangattacheditem service")
			@Test
			void shouldCallVorgangAttachedItemService() {
				callCreateWiedervorlage();

				verify(vorgangAttachedItemService).createNewWiedervorlage(any(Wiedervorlage.class), eq(VorgangHeaderTestFactory.ID));
			}
		}

		@DisplayName("Add created")
		@Nested
		class TestAddCreated {

			@BeforeEach
			void mockServices() {
				when(currentUserService.getUserId()).thenReturn(UserProfileTestFactory.ID);
			}

			@Test
			void shouldSetCreatedAt() throws Exception {
				var wiedervorlage = callAddCreated();

				assertThat(wiedervorlage.getCreatedAt()).isNotNull().isCloseTo(ZonedDateTime.now(), within(2, ChronoUnit.SECONDS));
			}

			@Test
			void shouldSetCreatedBy() throws Exception {
				var wiedervorlage = callAddCreated();

				assertThat(wiedervorlage.getCreatedBy()).isEqualTo(UserProfileTestFactory.ID.toString());
			}

			private Wiedervorlage callAddCreated() {
				return service.addCreated(WiedervorlageTestFactory.createBuilder().createdAt(null).createdBy(null).build());
			}
		}

		private Command callCreateWiedervorlage() {
			return service.createWiedervorlage(WiedervorlageTestFactory.create(), VorgangHeaderTestFactory.ID);
		}
	}

	@DisplayName("Edit wiedervorlage")
	@Nested
	class TestEditWiedervorlage {

		@Captor
		private ArgumentCaptor<Wiedervorlage> wiedervorlageCaptor;

		@Test
		void shouldCallVorgangAttachedItemService() {
			callEditWiedervorlage();

			verify(vorgangAttachedItemService).editWiedervorlage(any(Wiedervorlage.class), any(), anyLong());
		}

		private Command callEditWiedervorlage() {
			return service.editWiedervorlage(WiedervorlageTestFactory.create(), WiedervorlageTestFactory.ID, WiedervorlageTestFactory.VERSION);
		}
	}

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(WiedervorlageTestFactory.ID);

			verify(remoteService).getById(WiedervorlageTestFactory.ID);
		}
	}

	@Nested
	class TestFindByVorgangId {

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(remoteService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestUpdateNextFrist {

		private final Wiedervorlage wiedervorlage = WiedervorlageTestFactory.create();
		private final Wiedervorlage foundWiedervorlage = WiedervorlageTestFactory.create();

		@Captor
		private ArgumentCaptor<Stream<Wiedervorlage>> allWiedervorlagen;

		@BeforeEach
		void mockService() {
			when(remoteService.findByVorgangId(VorgangHeaderTestFactory.ID)).thenReturn(Stream.of(foundWiedervorlage));
			doReturn(Optional.of(WiedervorlageTestFactory.FRIST)).when(service).calculateNextFrist(any());
		}

		@Test
		void shouldDoCalculation() {
			callUpdateNextFrist();

			verify(service).calculateNextFrist(any());
		}

		@Test
		void shouldAddWiedervorlageToFoundWiedervorlagen() {
			callUpdateNextFrist();

			verify(service).calculateNextFrist(allWiedervorlagen.capture());
			assertThat(allWiedervorlagen.getValue()).containsExactly(wiedervorlage);
		}

		@Test
		void shouldCallFindByVorgangId() {
			callUpdateNextFrist();

			verify(service).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallRemoteService() {
			callUpdateNextFrist();

			verify(remoteService).updateNextFrist(VorgangHeaderTestFactory.ID, Optional.of(WiedervorlageTestFactory.FRIST));
		}

		private void callUpdateNextFrist() {
			service.updateNextFrist(VorgangHeaderTestFactory.ID, wiedervorlage);
		}
	}

	@Nested
	class TestCalculateNextFrist {

		@Test
		void shouldReturnNullOnAllDone() {
			var wiedervorlage = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().plus(1, ChronoUnit.DAYS)).done(true).build();

			var nextFrist = calculateNextFrist(Stream.of(wiedervorlage));

			assertThat(nextFrist).isEmpty();
		}

		@Test
		void shouldReturnEarliestFrist() {
			var fristPast2Days = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().minus(2, ChronoUnit.DAYS)).done(false)
					.build();
			var fristPast1Day = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().minus(1, ChronoUnit.DAYS)).done(false).build();
			var fristFuture1Day = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().plus(1, ChronoUnit.DAYS)).done(false)
					.build();
			var fristFuture2Days = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().plus(2, ChronoUnit.DAYS)).done(false)
					.build();

			var nextFrist = calculateNextFrist(Stream.of(fristPast2Days, fristPast1Day, fristFuture1Day, fristFuture2Days));

			assertThat(nextFrist).contains(LocalDate.now().minus(2, ChronoUnit.DAYS));
		}

		@Test
		void shouldReturnFristIgnoringDone() {
			var fristPast1DayNotDone = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().plus(1, ChronoUnit.DAYS)).done(false)
					.build();
			var fristPast1DayDone = WiedervorlageTestFactory.createBuilder().frist(LocalDate.now().minus(1, ChronoUnit.DAYS)).done(true)
					.build();

			var nextFrist = calculateNextFrist(Stream.of(fristPast1DayNotDone, fristPast1DayDone));

			assertThat(nextFrist).contains(LocalDate.now().plus(1, ChronoUnit.DAYS));
		}

		private Optional<LocalDate> calculateNextFrist(Stream<Wiedervorlage> wiedervorlagen) {
			return service.calculateNextFrist(wiedervorlagen);
		}
	}
}