package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.HistorienProtokollInformationTypeTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangTypeTestFactory;
import de.xoev.xdomea.AbgabeAbgabe0401;
import de.xoev.xdomea.AkteType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.VorgangType;

class XdomeaNachrichtBuilderTest {

	private final VorgangType vorgangType = VorgangTypeTestFactory.create();
	@Spy
	private final XdomeaNachrichtBuilder builder = XdomeaNachrichtBuilder.builder().withVorgang(vorgangType);

	@Nested
	class TestBuild {
		private final AbgabeAbgabe0401 expectedAbgabe = AbgabeAbgabe0401TestFactory.create();

		@BeforeEach
		void setUpMock() {
			doReturn(expectedAbgabe).when(builder).createAbgabeType();
		}

		@Test
		void shouldCallAddVorgangDokumente() {
			builder.build();

			verify(builder).addVorgangDokumente();
		}

		@Test
		void shouldCallAddVorgangChangeHistory() {
			builder.build();

			verify(builder).addVorgangChangeHistory();
		}

		@Test
		void shouldCallCreateAbgabeType() {
			builder.build();

			verify(builder).createAbgabeType();
		}

		@Test
		void shouldReturnAbgabeType() {
			var abgabeType = builder.build();

			assertThat(abgabeType).isEqualTo(expectedAbgabe);
		}
	}

	@Nested
	class TestCreateAbgabeType {
		private final NkAbgabeType kopfType = NkAbgabeTypeTestFactory.create();
		private final AkteType akte = AkteTypeTestFactory.create();

		@Test
		void shouldHaveKopf() {
			var abgabeType = builder.withKopf(kopfType).createAbgabeType();

			assertThat(abgabeType.getKopf()).isEqualTo(kopfType);
		}

		@Test
		void shoulAddTwoSchriftgutobjekt() {
			var abgabeType = builder.withAktenzeichen(akte).createAbgabeType();

			assertThat(abgabeType.getSchriftgutobjekt()).size().isEqualTo(2);
		}

		@Test
		void shouldHaveVorgangSchriftgutObjekt() {
			var abgabeType = builder.createAbgabeType();

			assertThat(abgabeType.getSchriftgutobjekt().get(0).getVorgang()).isEqualTo(vorgangType);
		}

		@Test
		void shouldHaveAkteSchriftgutObjekt() {
			var abgabeType = builder.withAktenzeichen(akte).createAbgabeType();

			assertThat(abgabeType.getSchriftgutobjekt().get(1).getAkte()).isEqualTo(akte);
		}

	}

	@Nested
	class TestAddVorgangDokumente {
		private final List<DokumentType> representations = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
		private final List<DokumentType> attachments = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
		private final List<DokumentType> kommentare = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
		private final List<DokumentType> postfachMails = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());
		private final List<DokumentType> bescheids = List.of(DokumentTypeTestFactory.create(), DokumentTypeTestFactory.create());

		@Test
		void shouldAddRepresentations() {
			builder.withRepresentations(representations).addVorgangDokumente();

			assertThat(vorgangType.getDokument()).isEqualTo(representations);
		}

		@Test
		void shouldAddAttachments() {
			builder.withAttachments(attachments).addVorgangDokumente();

			assertThat(vorgangType.getDokument()).isEqualTo(attachments);
		}

		@Test
		void shouldAddKommentare() {
			builder.withKommentare(kommentare).addVorgangDokumente();

			assertThat(vorgangType.getDokument()).isEqualTo(kommentare);
		}

		@Test
		void shouldAddPostfachMails() {
			builder.withPostfachMails(postfachMails).addVorgangDokumente();

			assertThat(vorgangType.getDokument()).isEqualTo(postfachMails);
		}

		@Test
		void shouldAddBescheids() {
			builder.withBescheids(bescheids).addVorgangDokumente();

			assertThat(vorgangType.getDokument()).isEqualTo(bescheids);
		}

	}

	@Nested
	class TestAddVorgangChangeHistory {

		private final List<HistorienProtokollInformationType> historie = List.of(HistorienProtokollInformationTypeTestFactory.create(),
				HistorienProtokollInformationTypeTestFactory.create());

		@Test
		void shouldAddHistorie() {
			builder.withHistorie(historie).addVorgangChangeHistory();

			assertThat(vorgangType.getHistorienProtokollInformation()).isEqualTo(historie);
		}

		@Test
		void shouldNotAddHistorie() {
			builder.addVorgangChangeHistory();

			assertThat(vorgangType.getHistorienProtokollInformation()).isEmpty();
		}
	}

}
