package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.export.DokumentTypeTestFactory;
import de.ozgcloud.alfa.kommentar.KommentarsExportData.KommentarsExportDataBuilder;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import de.xoev.xdomea.DokumentType;

class ExportKommentarServiceTest {

	@Spy
	@InjectMocks
	private ExportKommentarService service;

	@Mock
	private KommentarService kommentarService;
	@Mock
	private BinaryFileService binaryFileService;

	@Mock
	private UserService userService;

	private final Kommentar kommentar = KommentarTestFactory.create();

	@Nested
	class TestCreateExportData {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final KommentarsExportData exportData = KommentarsExportDataTestFactory.create();

		private final ExportKommentarService.KommentarExportData kommentarExportData = new ExportKommentarService.KommentarExportData(
				ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID, kommentar);

		private MockedStatic<KommentarsExportData> kommentarsExportDataMockedStatic;
		@Mock
		private KommentarsExportDataBuilder exportDataBuilder;

		@BeforeEach
		void setUp() {
			kommentarsExportDataMockedStatic = mockStatic(KommentarsExportData.class);
			kommentarsExportDataMockedStatic.when(KommentarsExportData::builder).thenReturn(exportDataBuilder);
			when(service.getKommentare(vorgang)).thenReturn(Stream.of(kommentar));
			when(service.createKommentarExportData(vorgang, kommentar)).thenReturn(kommentarExportData);
			doNothing().when(service).addKommentarExportData(kommentarExportData, exportDataBuilder);
			when(exportDataBuilder.build()).thenReturn(exportData);
		}

		@AfterEach
		void tearDown() {
			kommentarsExportDataMockedStatic.close();
		}

		@Test
		void shouldAddKommentarExportData() {
			callService();

			verify(service).addKommentarExportData(eq(kommentarExportData), eq(exportDataBuilder));
		}

		@Test
		void shouldBuildExportData() {
			callService();

			verify(exportDataBuilder).build();
		}

		@Test
		void shouldReturnBuiltExportData() {
			var result = callService();

			assertThat(result).isEqualTo(exportData);
		}

		private KommentarsExportData callService() {
			return service.createExportData(vorgang);
		}
	}

	@Nested
	class TestAddKommentarExportData {

		private final String authorFullName = UserProfileTestFactory.FULLNAME;
		private final List<OzgFile> attachments = List.of(KommentarsExportDataTestFactory.OZG_FILE);
		private final DokumentType dokumentType = DokumentTypeTestFactory.create();
		private final String organisationsEinheitenID = ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID;
		private final ExportKommentarService.KommentarExportData kommentarExportData = new ExportKommentarService.KommentarExportData(
				organisationsEinheitenID,
				kommentar
		);
		private MockedStatic<DokumentTypeBuilder> dokumentTypeBuilderMockedStatic;
		@Mock
		private DokumentTypeBuilder dokumentTypeBuilder;
		@Mock
		private KommentarsExportDataBuilder exportDataBuilder;

		@BeforeEach
		void setUp() {
			doReturn(attachments).when(service).getAttachments(kommentar);
			doReturn(authorFullName).when(service).getAuthorFullName(kommentar);

			dokumentTypeBuilderMockedStatic = mockStatic(DokumentTypeBuilder.class);
			dokumentTypeBuilderMockedStatic.when(() -> DokumentTypeBuilder.builder()).thenReturn(dokumentTypeBuilder);

			doReturn(dokumentTypeBuilder).when(dokumentTypeBuilder).withKommentarAttachments(attachments);
			doReturn(dokumentTypeBuilder).when(dokumentTypeBuilder).withKommentar(kommentar);
			doReturn(dokumentTypeBuilder).when(dokumentTypeBuilder)
					.withOrganisationseinheitenID(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
			doReturn(dokumentTypeBuilder).when(dokumentTypeBuilder).withAuthorFullName(authorFullName);
			doReturn(dokumentType).when(dokumentTypeBuilder).build();
		}

		@AfterEach
		void tearDown() {
			dokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetDokumentType() {
			callService();

			verify(exportDataBuilder).dokumentType(dokumentType);
		}

		@Test
		void shouldSetAttachments() {
			callService();

			verify(exportDataBuilder).attachments(attachments);
		}

		@Test
		void shouldSetOzgFile() {
			callService();

			verify(dokumentTypeBuilder).withKommentarAttachments(attachments);
		}

		@Test
		void shouldSetKommentar() {
			callService();

			verify(dokumentTypeBuilder).withKommentar(kommentar);
		}

		@Test
		void shouldSetOrganisationseinheitID() {
			callService();

			verify(dokumentTypeBuilder).withOrganisationseinheitenID(organisationsEinheitenID);
		}

		@Test
		void shouldReturnAuthorFullName() {
			callService();

			verify(dokumentTypeBuilder).withAuthorFullName(authorFullName);
		}

		@Test
		void shouldBuildDokumentType() {
			callService();

			verify(dokumentTypeBuilder).build();
		}

		private void callService() {
			service.addKommentarExportData(kommentarExportData, exportDataBuilder);
		}
	}

	@Nested
	class TestGetAttachments {

		@Test
		void shouldGetFiles() {
			service.getAttachments(kommentar);

			verify(binaryFileService).getFiles(KommentarTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldReturnAttachments() {
			List<OzgFile> attachments = List.of(KommentarsExportDataTestFactory.OZG_FILE);
			when(binaryFileService.getFiles(KommentarTestFactory.ATTACHMENTS)).thenReturn(attachments.stream());

			var result = service.getAttachments(kommentar);

			assertThat(result).isEqualTo(attachments);
		}
	}

	@Nested
	class TestGetAuthorFullName {

		@BeforeEach
		void init() {
			when(userService.getById(UserProfileTestFactory.ID)).thenReturn(UserProfileTestFactory.create());
		}

		@Test
		void shouldGetUser() {
			service.getAuthorFullName(KommentarTestFactory.create());

			verify(userService).getById(eq(UserId.from(KommentarTestFactory.CREATED_BY)));
		}

		@Test
		void shouldReturnAuthorsFullName() {
			var authorFullName = service.getAuthorFullName(KommentarTestFactory.create());

			assertThat(authorFullName).isEqualTo(UserProfileTestFactory.FULLNAME);
		}
	}

	@Nested
	class TestGetKommentare {

		private VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@Test
		void shouldFindKommentareByVorgang() {
			service.getKommentare(vorgang);

			verify(kommentarService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnKommentare() {
			when(kommentarService.findByVorgangId(VorgangHeaderTestFactory.ID)).thenReturn(Stream.of(kommentar));

			var result = service.getKommentare(vorgang);

			assertThat(result).containsExactly(kommentar);
		}
	}

}