package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.alfa.common.AnlageDokumentTypeTestFactory;
import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.HistorienProtokollInformationTypeTestFactory;
import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.IdentifikationObjektType;

public class DokumentTypeBuilderTest {

	private final PostfachMail postfachMail = PostfachMailTestFactory.create();
	private final OzgFile ozgFile = OzgFileTestFactory.createBuilder().id(BinaryFileTestFactory.FILE_ID).build();

	@Spy
	@InjectMocks
	private final DokumentTypeBuilder builder = DokumentTypeBuilder.builder()
			.withPostfachMail(postfachMail)
			.withOzgFiles(List.of(ozgFile))
			.withOrganisationseinheitenId(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

	@Nested
	class TestBuild {
		private static final String TYP = "Nachricht";

		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		private final AnlageDokumentType anlageDokument = AnlageDokumentTypeTestFactory.create();
		private final HistorienProtokollInformationType historienProtokollInformation = HistorienProtokollInformationTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(PostfachMailTestFactory.ID)).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);

			doReturn(historienProtokollInformation).when(builder).createHistorienProtokollInformation();
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
		}

		@Nested
		class TestWithOneAnlage {
			@BeforeEach
			void mockCreateAnlage() {
				doReturn(anlageDokument).when(builder).createAnlage(ozgFile);
			}

			@Test
			void shouldSetObjectId() {
				builder.build();

				verify(identifikationObjektTypeBuilder).withObjectID(postfachMail.getId());
			}

			@Test
			void shouldBuildIdentifikationObjectType() {
				builder.build();

				verify(identifikationObjektTypeBuilder).build();
			}

			@Test
			void shouldHaveIdentifikation() {
				var dokument = builder.build();

				assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjektType);
			}

			@Test
			void shouldHaveTyp() {
				var dokument = builder.build();

				assertThat(dokument.getTyp()).isEqualTo(TYP);
			}

			@Test
			void shouldHaveHistorienProtokollInformation() {
				var dokument = builder.build();

				assertThat(dokument.getHistorienProtokollInformation()).containsExactly(historienProtokollInformation);
			}

			@Test
			void shouldCallCreateAnlage() {
				builder.build();

				verify(builder).createAnlage(ozgFile);
			}

			@Test
			void shouldHaveOneAnlage() {
				var dokument = builder.build();

				assertThat(dokument.getAnlage()).containsExactly(anlageDokument);
			}
		}

		@Nested
		class TestWithoutAnlage {
			@Test
			void shouldHaveNoAnlage() {
				var dokument = builder.withOzgFiles(Collections.emptyList()).build();

				assertThat(dokument.getAnlage()).isEmpty();
			}
		}
	}

	@Nested
	class TestCreateHistorienProtokollInformation {
		private static final String AKTEUR = "correct aktuer";
		private static final String NACHRICHT_EMPFANGEN = "Nachricht empfangen";
		private static final String NACHRICHT_GESENDET = "Nachricht gesendet";

		@BeforeEach
		void setUpMock() {
			doReturn(AKTEUR).when(builder).getAkteur();
			doReturn(PostfachMailTestFactory.CREATED_AT).when(builder).getSentTime();
		}

		@Test
		void shouldHaveMetadatumName() {
			var resultHistorie = callCreateHistorienProtokollInformation();

			assertThat(resultHistorie.getMetadatumName()).isEqualTo(PostfachMailTestFactory.MAIL_BODY);
		}

		@Test
		void shouldHaveAkteur() {
			var resultHistorie = callCreateHistorienProtokollInformation();

			assertThat(resultHistorie.getAkteur()).isEqualTo(AKTEUR);
		}

		@Test
		void shouldCallGetSentTime() {
			callCreateHistorienProtokollInformation();

			verify(builder).getSentTime();
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var resultHistorie = callCreateHistorienProtokollInformation();

			assertThat(resultHistorie.getDatumUhrzeit()).isEqualTo(DateConverter.toXmlGregorianCalendar(PostfachMailTestFactory.CREATED_AT));
		}

		@Test
		void shouldReturnAktionWithGesendet() {
			builder.withPostfachMail(PostfachMailTestFactory.createBuilder().direction(Direction.OUT).build());

			var resultHistorie = callCreateHistorienProtokollInformation();

			assertThat(resultHistorie.getAktion()).isEqualTo(NACHRICHT_GESENDET);
		}

		@Test
		void shouldReturnAktionWithEmpfangen() {
			builder.withPostfachMail(PostfachMailTestFactory.createBuilder().direction(Direction.IN).build());

			var resultHistorie = callCreateHistorienProtokollInformation();

			assertThat(resultHistorie.getAktion()).isEqualTo(NACHRICHT_EMPFANGEN);
		}

		private HistorienProtokollInformationType callCreateHistorienProtokollInformation() {
			return builder.createHistorienProtokollInformation();
		}
	}

	@Nested
	class TestGetAkteur {
		private final UserProfile userProfile = UserProfileTestFactory.create();

		@Test
		void shouldReturnNameAndOrgaIdOfBearbeiter() {
			builder.withUserProfile(userProfile);

			var akteur = builder.getAkteur();

			assertThat(akteur).isEqualTo(UserProfileTestFactory.FULLNAME + "; " + ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
		}

		@Test
		void shouldReturnAntragsteller() {
			builder.withUserProfile(null);

			var akteur = builder.getAkteur();

			assertThat(akteur).isEqualTo("Antragsteller");
		}
	}

	@Nested
	class TestCreateAnlage {
		private MockedStatic<AnlageDokumentTypeBuilder> anlageDokumentTypeBuilderMockedStatic;
		@Mock
		private AnlageDokumentTypeBuilder anlageDokumentTypeBuilder;
		private AnlageDokumentType expectedAnlage = AnlageDokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			anlageDokumentTypeBuilderMockedStatic = mockStatic(AnlageDokumentTypeBuilder.class);
			anlageDokumentTypeBuilderMockedStatic.when(AnlageDokumentTypeBuilder::builder).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.withOzgFile(ozgFile)).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.build()).thenReturn(expectedAnlage);
		}

		@AfterEach
		void tearDown() {
			anlageDokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetOzgFile() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldBuildAnlageDokumentType() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).build();
		}

		@Test
		void shouldReturnBuiltAnlage() {
			var resultAnlage = callCreateAnlage();

			assertThat(resultAnlage).isEqualTo(expectedAnlage);
		}

		private AnlageDokumentType callCreateAnlage() {
			return builder.createAnlage(ozgFile);
		}
	}

	@Nested
	class TestGetSentTime {
		private final ZonedDateTime testTime = ZonedDateTime.now().withNano(0);

		@Test
		void shouldReturnCreatedAtForIncoming() {
			builder.withPostfachMail(PostfachMailTestFactory.createBuilder().direction(Direction.IN).createdAt(testTime).build());

			var resultTime = builder.getSentTime();

			assertThat(resultTime).isEqualTo(testTime);
		}

		@Test
		void shouldReturnSentAtForOutgoing() {
			builder.withPostfachMail(PostfachMailTestFactory.createBuilder().direction(Direction.OUT).sentAt(testTime).build());

			var resultTime = builder.getSentTime();

			assertThat(resultTime).isEqualTo(testTime);
		}
	}
}
