package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.export.XdomeaPropertiesTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.export.XdomeaProperties;
import de.xoev.xdomea.BehoerdenkennungType;
import de.xoev.xdomea.Code;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NachrichtentypCodeType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.OrganisationseinheitType;
import de.xoev.xdomea.SystemType;

class KopfCreatorTest {

	@Spy
	@InjectMocks
	private KopfCreator creator;

	@Mock
	private XdomeaProperties xdomeaProperties;

	@Nested
	class TestCreateKopf {

		static final ZonedDateTime CREATION_TIME = ZonedDateTime.now();

		private MockedStatic<UUIDConverter> mockedUUIDConverter;

		@BeforeEach
		void setUpUUIDMock() {
			mockedUUIDConverter = mockStatic(UUIDConverter.class);
		}

		@AfterEach
		void tearDown() {
			mockedUUIDConverter.close();
		}

		@Test
		void shouldSetProcessId() {
			var uuid = LoremIpsum.getInstance().getWords(1);
			mockedUUIDConverter.when(() -> UUIDConverter.fromObjectId(VorgangHeaderTestFactory.ID)).thenReturn(uuid);

			var kopf = createKopf();

			assertThat(kopf.getProzessID()).isEqualTo(uuid);
		}

		@Test
		void shouldSetNachrichtentyp() {
			var expectedValue = new NachrichtentypCodeType();
			doReturn(expectedValue).when(creator).createNachrichtentyp();

			var kopf = createKopf();

			assertThat(kopf.getNachrichtentyp()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetErstellungszeitpunkt() {
			try (var zonedDateTime = mockStatic(ZonedDateTime.class); var dateConverter = mockStatic(DateConverter.class)) {
				zonedDateTime.when(() -> ZonedDateTime.now(ZoneOffset.UTC)).thenReturn(CREATION_TIME);

				var expectedValue = mock(XMLGregorianCalendar.class);
				dateConverter.when(() -> DateConverter.toXmlGregorianCalendar(CREATION_TIME)).thenReturn(expectedValue);

				var kopf = createKopf();

				assertThat(kopf.getErstellungszeitpunkt()).isEqualTo(expectedValue);
			}
		}

		@Test
		void shouldSetAbsender() {
			var expectedValue = new KontaktType();
			doReturn(expectedValue).when(creator).createAbsender(any());

			var kopf = createKopf();

			assertThat(kopf.getAbsender()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetEmpfaenger() {
			var expectedValue = new KontaktType();
			doReturn(expectedValue).when(creator).createKontaktType();

			var kopf = createKopf();

			assertThat(kopf.getEmpfaenger()).isEqualTo(expectedValue);
		}

		@Test
		void shouldSetSendendesSystem() {
			var sendendesSystem = new SystemType();
			doReturn(sendendesSystem).when(creator).createSendendesSystem();

			var kopf = createKopf();

			assertThat(kopf.getSendendesSystem()).isEqualTo(sendendesSystem);
		}

		@Test
		void shouldSetImportbestaetigung() {
			var kopf = createKopf();

			assertThat(kopf.isImportbestaetigung()).isTrue();
		}

		@Test
		void shouldSetEmpfangsbestaetigung() {
			var kopf = createKopf();

			assertThat(kopf.isEmpfangsbestaetigung()).isTrue();
		}

		private NkAbgabeType createKopf() {
			return creator.createKopf(VorgangWithEingangTestFactory.create());
		}
	}

	@Nested
	class TestCreateNachrichtentype {

		@Test
		void shouldSetCode() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getCode()).isEqualTo(KopfCreator.NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE);
		}

		@Test
		void shouldSetListURI() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getListURI()).isEqualTo(KopfCreator.NACHRICHTENTYP_CODE_TYPE_LIST_URI);
		}

		@Test
		void shouldSetListVersionID() {
			var nachrichtentyp = creator.createNachrichtentyp();

			assertThat(nachrichtentyp.getListVersionID()).isEqualTo(KopfCreator.NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID);
		}
	}

	@Nested
	class TestCreateAbsender {

		@Test
		void shouldCallCreateKontaktType() {
			creator.createAbsender(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			verify(creator).createKontaktType();
		}

		@Test
		void shouldSetOrganisationseinheitType() {
			var expectedValue = new OrganisationseinheitType();
			doReturn(expectedValue).when(creator).createOrganisationseinheitType(any());

			var absender = creator.createAbsender(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(absender.getOrganisationseinheit()).isEqualTo(expectedValue);
		}
	}

	@Nested
	class TestCreateKontaktType {

		@Nested
		class OnBehoerdenschluesselNotSet {

			@ParameterizedTest
			@NullAndEmptySource
			void shouldNotCreateBehoerdenkennung(String behoerdenschluessel) {
				when(xdomeaProperties.getBehoerdenschluessel()).thenReturn(behoerdenschluessel);

				creator.createKontaktType();

				verify(creator, never()).createBehoerdenkennung();
			}

			@ParameterizedTest
			@NullAndEmptySource
			void shouldNotSetBehoerdenkennung(String behoerdenschluessel) {
				when(xdomeaProperties.getBehoerdenschluessel()).thenReturn(behoerdenschluessel);

				var kontaktType = creator.createKontaktType();

				assertThat(kontaktType.getBehoerdenkennung()).isNull();
			}
		}

		@Nested
		class OnBehoerdenschluesselSet {

			private static final BehoerdenkennungType expectedValue = new BehoerdenkennungType();

			@BeforeEach
			void init() {
				when(xdomeaProperties.getBehoerdenschluessel()).thenReturn(BEHOERDENSCHLUESSEL);
				doReturn(expectedValue).when(creator).createBehoerdenkennung();
			}

			@Test
			void shouldCreateBehoerdenkennung() {
				creator.createKontaktType();

				verify(creator).createBehoerdenkennung();
			}

			@Test
			void shouldSetBehoerdenkennung() {
				var kontaktType = creator.createKontaktType();

				assertThat(kontaktType.getBehoerdenkennung()).isEqualTo(expectedValue);
			}
		}
	}

	@Nested
	class TestCreateOrganisationseinheitType {

		@Test
		void shouldSetName() {
			var organisationseinheitType = creator.createOrganisationseinheitType(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(organisationseinheitType.getName()).isEqualTo(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
		}
	}

	@Nested
	class TestCreateBehoerdenkennung {

		private final Code expectedBehoerdenschluessel = new Code();

		@BeforeEach
		void init() {
			doReturn(expectedBehoerdenschluessel).when(creator).createBehoerdenschluessel();
		}

		@Test
		void shouldcreateBehoerdenschluessel() {
			creator.createBehoerdenkennung();

			verify(creator).createBehoerdenschluessel();
		}

		@Test
		void shouldSetBehoerdenschluessel() {
			var behoerdenkennungType = creator.createBehoerdenkennung();

			assertThat(behoerdenkennungType.getBehoerdenschluessel()).isEqualTo(expectedBehoerdenschluessel);
		}
	}

	@Nested
	class TestCreateBehoerdenschlussel {

		@BeforeEach
		void init() {
			when(xdomeaProperties.getBehoerdenschluessel()).thenReturn(BEHOERDENSCHLUESSEL);
			when(xdomeaProperties.getBehoerdenschluesselUri()).thenReturn(BEHOERDENSCHLUESSEL_URI);
			when(xdomeaProperties.getBehoerdenschluesselVersion()).thenReturn(BEHOERDENSCHLUESSEL_VERSION);
		}

		@Test
		void shouldSetCode() {
			var behoerdenschlussel = creator.createBehoerdenschluessel();

			assertThat(behoerdenschlussel.getCode()).isEqualTo(BEHOERDENSCHLUESSEL);
		}

		@Test
		void shouldSetListURI() {
			var behoerdenschlussel = creator.createBehoerdenschluessel();

			assertThat(behoerdenschlussel.getListURI()).isEqualTo(BEHOERDENSCHLUESSEL_URI);
		}

		@Test
		void shouldSetListVersionID() {
			var behoerdenschlussel = creator.createBehoerdenschluessel();

			assertThat(behoerdenschlussel.getListVersionID()).isEqualTo(BEHOERDENSCHLUESSEL_VERSION);
		}
	}

	@Nested
	class TestCreateSendendesSystem {

		@Test
		void shouldSetSystemName() {
			var sendendesSystem = creator.createSendendesSystem();

			assertThat(sendendesSystem.getProduktname()).isEqualTo(KopfCreator.PRODUKT_NAME);
		}
	}
}